/* This function clips a line using its two endpoint coordinates (x1, y1) & (x2, y2) */
int CohenSutherlandClip(float &x1, float &y1, float &x2, float &y2)
{
    //Global variables specifying the clipping codes
	#define CLIPPING_REGION_OUTCODE 0x0000 //binary: 0000
	#define TOP_CENTRE_OUTCODE		0x0008 //binary: 1000
	#define BOTTOM_CENTRE_OUTCODE	0x0004 //binary: 0100 
	#define MIDDLE_RIGHT_OUTCODE	0x0002 //binary: 0010
	#define MIDDLE_LEFT_OUTCODE		0x0001 //binary: 0001
	#define TOP_RIGHT_OUTCODE		0x000a //binary: 1010
	#define BOTTOM_RIGHT_OUTCODE	0x0006 //binary: 0110
	#define TOP_LEFT_OUTCODE		0x0009 //binary: 1001
	#define BOTTOM_LEFT_OUTCODE		0x0005 //binary: 0101

	/* assign the received endpoint coordinates local coordinates */

	int x1_clipping_coordinate = x1;
	int y1_clipping_coordinate = y1;
	int x2_clipping_coordinate = x2;
	int y2_clipping_coordinate = y2;

	/* specify the clipping rectangle borders. 
	   - the clipping edges are given the same names as those shown in Figure 12.16  */
	int x_min = 0;
	int x_max = SCREEN_WIDTH;
	int y_min = 0;
	int y_max = SCREEN_HEIGHT;

	int line_segment_endpoint1 = 0;
	int line_segment_endpoint2 = 0;

	/* use the bitwise inclusive OR and assign operator |= to assign outcodes to each of the line segment endpoints */
	
	/////////////////////////////////////////////////
	/* determine the outcode of the first endpoint */
	/////////////////////////////////////////////////
	if (x1 < x_min)
		line_segment_endpoint1 |= MIDDLE_LEFT_OUTCODE;
	else
		if (x1 > x_max)
			line_segment_endpoint1 |= MIDDLE_RIGHT_OUTCODE;

	if (y1 < y_min)
		line_segment_endpoint1 |= TOP_CENTRE_OUTCODE;
	else
		if (y1 > y_max)
			line_segment_endpoint1 |= BOTTOM_CENTRE_OUTCODE;

	//////////////////////////////////////////////////
	/* determine the outcode of the second endpoint */
	//////////////////////////////////////////////////
	if (x2 < x_min)
		line_segment_endpoint2 |= MIDDLE_LEFT_OUTCODE;
	else
		if (x2 > x_max)
			line_segment_endpoint2 |= MIDDLE_RIGHT_OUTCODE;

	if (y2 < y_min)
		line_segment_endpoint2 |= TOP_CENTRE_OUTCODE;
	else
		if (y2 > y_max)
			line_segment_endpoint2 |= BOTTOM_CENTRE_OUTCODE;

	/* both endpoints will be equal to "0" if the line is completely visible */
	if (line_segment_endpoint1 == 0 && line_segment_endpoint2 == 0)
		return 0;

	//////////////////////////////////////////////////
	/* clip the line for the first endpoint using the defined clipping window and determined outcode */
	//////////////////////////////////////////////////
	switch(line_segment_endpoint1)
	{
		/* the endpoint is located within the clipping window */
		case CLIPPING_REGION_OUTCODE:
		break;

		/* the endpoint is located within the bottom-centre clipping region */
		case BOTTOM_CENTRE_OUTCODE:
		{
			int m = (x2 - x1)/(y2 - y1);
			x1_clipping_coordinate = x1 + 0.5 + (y_max - y1) * m;
			y1_clipping_coordinate = y_max;
		}
		break;

		/* the endpoint is located within the top-centre clipping region */
		case TOP_CENTRE_OUTCODE:
		{
			int m = (x2 - x1)/(y2 - y1);
			
			x1_clipping_coordinate = x1 + 0.5 + (y_min - y1) * m;
			y1_clipping_coordinate = y_min;

		}
		break;

		/* the endpoint is located within the middle-right clipping region */
		case MIDDLE_RIGHT_OUTCODE:
		{
			int m = (y2 - y1)/(x2 - x1);

			y1_clipping_coordinate = y1 + 0.5 + (x_max - x1) * m;
			x1_clipping_coordinate = x_max;
		}
		break;

		/* the endpoint is located within the middle-left clipping region */
		case MIDDLE_LEFT_OUTCODE:
		{
			int m = (y2 - y1)/(x2 - x1);

			y1_clipping_coordinate = y1 + 0.5 + (x_min - x1) * m;
			x1_clipping_coordinate = x_min;

		}
		break;

		/* the endpoint is located within the bottom-right clipping region, requiring the computation of two intersections */
		case BOTTOM_RIGHT_OUTCODE:
		{
			int m = (x2 - x1)/(y2 - y1);

			/* calculate the bottom horizontal line intersection*/
			x1_clipping_coordinate = x1 + 0.5 + (y_max - y1) * m;
			y1_clipping_coordinate = y_max;

			/* the horizontal line intersection is tested for validity, if it passes then we can calculate the right vertical line intersection */
			if (x1_clipping_coordinate > x_max || x1_clipping_coordinate < x_min)
			{
				int m2 = (y2 - y1)/(x2 - x1);

				y1_clipping_coordinate = y1 + 0.5 + (x_max - x1) * m2;
				x1_clipping_coordinate = x_max;
			}
		}
		break;

		/* the endpoint is located within the top-right clipping region, requiring the computation of two intersections */
		case TOP_RIGHT_OUTCODE:
		{
			int m = (x2 - x1)/(y2 - y1);

			/* calculate the top horizontal line intersection*/
			x1_clipping_coordinate = x1 + 0.5 + (y_min - y1) * m;
			y1_clipping_coordinate = y_min;

			/* the horizontal line intersection is tested for validity, if it passes then we can calculate the right vertical line intersection */
			if (x1_clipping_coordinate > x_max || x1_clipping_coordinate < x_min)
			{
				int m2 = (y2 - y1)/(x2 - x1);

				y1_clipping_coordinate = y1 + 0.5 + (x_max - x1) * m2;
				x1_clipping_coordinate = x_max;
			}
		}
		break;

		/* the endpoint is located within the bottom-left clipping region, requiring the computation of two intersections */
		case BOTTOM_LEFT_OUTCODE:
		{
			int m = (x2 - x1)/(y2 - y1);

			/* calculate the bottom horizontal line intersection*/
			x1_clipping_coordinate = x1 + 0.5 + (y_max - y1) * (x2 - x1)/(y2 - y1);
			y1_clipping_coordinate = y_max;

			/* the horizontal line intersection is tested for validity, if it passes then we can calculate the left vertical line intersection */
			if (x1_clipping_coordinate > x_max || x1_clipping_coordinate < x_min)
			{
				int m2 = (y2 - y1)/(x2 - x1);

				y1_clipping_coordinate = y1 + 0.5 + (x_min - x1) * m2;
				x1_clipping_coordinate = x_min;
			}
		}
		break;

		/* the endpoint is located within the top-left clipping region, requiring the computation of two intersections */
		case TOP_LEFT_OUTCODE:
		{
			int m = (x2 - x1)/(y2 - y1);

			/* calculate the top horizontal line intersection*/
			x1_clipping_coordinate = x1 + 0.5 + (y_min - y1) * (x2 - x1)/(y2 - y1);
			y1_clipping_coordinate = y_min;

			/* the horizontal line intersection is tested for validity, if it passes then we can calculate the left vertical line intersection */
			if (x1_clipping_coordinate > x_max || x1_clipping_coordinate < x_min)
			{
				int m2 = (y2 - y1)/(x2 - x1);

				y1_clipping_coordinate = y1 + 0.5 + (x_min - x1) * m2;
				x1_clipping_coordinate = x_min;
			}
		}
		break;

		default:break;
	}

	//////////////////////////////////////////////////
	/* clip the line for the second endpoint using the defined clipping window and determined outcode */
	//////////////////////////////////////////////////
	switch(line_segment_endpoint2)
	{
		/* the endpoint is located within the clipping window */
		case CLIPPING_REGION_OUTCODE:
		break;
		
		/* the endpoint is located within the bottom-centre clipping region */
		case BOTTOM_CENTRE_OUTCODE:
		{
			int n = (x1 - x2)/(y1 - y2);

			x2_clipping_coordinate = x2 + 0.5 + (y_max - y2) * n;
			y2_clipping_coordinate = y_max;
		}
		break;

		/* the endpoint is located within the top-centre clipping region */
		case TOP_CENTRE_OUTCODE:
		{
			int n = (x1 - x2)/(y1 - y2);

			x2_clipping_coordinate = x2 + 0.5 + (y_min - y2) * n;
			y2_clipping_coordinate = y_min;
		}
		break;

		/* the endpoint is located within the middle-right clipping region */
		case MIDDLE_RIGHT_OUTCODE:
		{
			int n = (y1 - y2)/(x1 - x2);

			y2_clipping_coordinate = y2 + 0.5 + (x_max - x2) * n;
			x2_clipping_coordinate = x_max;
		}
		break;

		/* the endpoint is located within the middle-left clipping region */
		case MIDDLE_LEFT_OUTCODE:
		{
			int n = (y1 - y2)/(x1 - x2);
			
			y2_clipping_coordinate = y2 + 0.5 + (x_min - x2) * n;
			x2_clipping_coordinate = x_min;
		}
		break;

		/* the endpoint is located within the bottom-right clipping region, requiring the computation of two intersections */
		case BOTTOM_RIGHT_OUTCODE:
		{
			int n = (x1 - x2)/(y1 - y2);

			/* calculate the bottom horizontal line intersection*/
			x2_clipping_coordinate = x2 + 0.5 + (y_max - y2) * n;
			y2_clipping_coordinate = y_max;

			/* the horizontal line intersection is tested for validity, if it passes then we can calculate the right vertical line intersection */
			if (x2_clipping_coordinate < x_min || x2_clipping_coordinate > x_max)
			{
				int n2 = (y1 - y2)/(x1 - x2);
				
				y2_clipping_coordinate = y2 + 0.5 + (x_max - x2) * n2;
				x2_clipping_coordinate = x_max;
			}
		}
		break;

		/* the endpoint is located within the top-right clipping region, requiring the computation of two intersections */
		case TOP_RIGHT_OUTCODE:
		{
			int n = (x1 - x2)/(y1 - y2);

			/* calculate the top horizontal line intersection*/
			x2_clipping_coordinate = x2 + 0.5 + (y_min - y2) * n;
			y2_clipping_coordinate = y_min;

			/* the horizontal line intersection is tested for validity, if it passes then we can calculate the right vertical line intersection */
			if (x2_clipping_coordinate < x_min || x2_clipping_coordinate > x_max)
			{
				int n2 = (y1 - y2)/(x1 - x2);
	
				y2_clipping_coordinate = y2 + 0.5 + (x_max - x2) * n2;
				x2_clipping_coordinate = x_max;
			}
		}
		break;

		/* the endpoint is located within the bottom-left clipping region, requiring the computation of two intersections */
		case BOTTOM_LEFT_OUTCODE:
		{
			int n = (x1 - x2)/(y1 - y2);

			/* calculate the bottom horizontal line intersection*/
			x2_clipping_coordinate = x2 + 0.5 + (y_max - y2) * n;
			y2_clipping_coordinate = y_max;

			/* the horizontal line intersection is tested for validity, if it passes then we can calculate the left vertical line intersection */
			if (x2_clipping_coordinate < x_min || x2_clipping_coordinate > x_max)
			{
				int n2 = (y1 - y2)/(x1 - x2);
				
				y2_clipping_coordinate = y2 + 0.5 + (x_min - x2) * n2;
				x2_clipping_coordinate = x_min;
			}
		}
		break;

		/* the endpoint is located within the top-left clipping region, requiring the computation of two intersections */
		case TOP_LEFT_OUTCODE:
		{
			int n = (x1 - x2)/(y1 - y2);

			/* calculate the top horizontal line intersection*/
			x2_clipping_coordinate = x2 + 0.5 + (y_min - y2) * n;
			y2_clipping_coordinate = y_min;

			/* the horizontal line intersection is tested for validity, if it passes then we can calculate the left vertical line intersection */
			if (x2_clipping_coordinate < x_min || x2_clipping_coordinate > x_max)
			{
				int n2 = (y1 - y2)/(x1 - x2);
				
				y2_clipping_coordinate = y2 + 0.5 + (x_min - x2) * n2;
				x2_clipping_coordinate = x_min;
			}
		}
		break;

		default:break;
	}

	/* return the clipping coordinates to the address variables */
	x1 = x1_clipping_coordinate;
	y1 = y1_clipping_coordinate;
	x2 = x2_clipping_coordinate;
	y2 = y2_clipping_coordinate'

	return 1;
}