//code to render a height map representing a hat function

#include <stdlib.h>
#include <GL/glut.h>
#include <math.h>

#define ESCAPE 27 //define ASCII value for special keystrokes

/* variable for g_horisontal rotation, used to rotate on keys Q and E */
GLfloat g_horisontal = 45.0;
/* variable for g_vertical rotation, used to rotate on keys W and S */
GLfloat g_vertical = 30.0;

/* resolution of heightmap */
const GLint g_mapDimension = 24;
/* window dimensions */
const GLint g_windowWidth = 640, g_windowHeight = 480;

/* matrix storing height values for heightmap */
GLfloat map[g_mapDimension][g_mapDimension];

/*
=================
setupHeights
-Samples the mathematical "hat" function at regular intervals
=================
*/
void setupHeights()
{
	double hatStartXPos = -6.0, hatStartZPos = -6.0, currentXPos = -6.0, currentZPos = -6.0;

	for (int i=0; i < g_mapDimension; i++)
	{
		currentZPos = hatStartZPos;

		for (int j=0; j < g_mapDimension; j++)
		{
			/* value generation function */
			map[i][j] = sin(sqrt((currentXPos)*(currentXPos) + (currentZPos)*(currentZPos))) / 2.0;
			currentZPos += 12.0 / g_mapDimension;
		}
		currentXPos += 12.0 / g_mapDimension;
	}
}

/*
=================
setColor
-Sets the colour based on the physical height positions
=================
*/
void setColor(GLfloat height)
{
	/* red */
	if(height < -0.2)
	{
		glColor3f(1.0, 0.0, 0.0);
		return;
	}
	/* green */
	else if(height < 0.2)
	{
		glColor3f(0.0, 0.8, 0.0);
		return;
	}
	/* blue */
	else
	{
		glColor3f(0.0, 0.0, 1.0);
		return;
	}
}

/*
=================
display
-Renders output to screen
=================
*/
void display(void)
{
	int i, j;

	/* set the clear colour - red, green, blue, alpha*/
	glClearColor(1.0, 1.0, 1.0, 0.0);

	/* clear frame buffer and z-buffer */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	/* used for scaling the heightmap, change the 2.0 to 4.0 to see effect */
	GLfloat scale = ((float)g_mapDimension/2.0);

	/* pushes current matrix stack down by one */
	glPushMatrix();   //saves current scalee
		/* positions the surface in front of the default camera */
		glTranslatef(0.0, 0.0, -3.5);

		/***rotations change on keyboard input***/
		/* initial rotation corresponds to isometric drawing angle (30 degrees around x-axis) */
		glRotatef(g_vertical, 1.0, 0.0, 0.0);
		/* initial rotation of 45 degrees around y-axis */
		glRotatef(g_horisontal, 0.0, 1.0,0.0);

		/* draw the white wireframe */
		glColor3f(1.0,1.0,1.0);
		/* sets the width of the wireframe lines */
		glLineWidth(2);
        for(i = 0; i < g_mapDimension - 1; i++)
		{
			for(j = 0; j< g_mapDimension - 1; j++)
			{
				glBegin(GL_LINE_LOOP);
					glVertex3f((i+1)/scale - 1.0, map[i+1][j], j/scale - 1.0);
					glVertex3f((i)/scale - 1.0, map[i][j], j/scale - 1.0);
					glVertex3f((i)/scale - 1.0, map[i][j+1], (j+1)/scale - 1.0);
					glVertex3f((i+1)/scale - 1.0, map[i+1][j], (j)/scale - 1.0);
					glVertex3f((i+1)/scale - 1.0, map[i+1][j+1], (j+1)/scale - 1.0);
					glVertex3f((i)/scale - 1.0, map[i][j+1], (j+1)/scale - 1.0);
				glEnd();
			}
		}

		/* draw the solid heightmap by running through the height values*/
		for(i = 0; i < g_mapDimension - 1; i++)
		{
			for(j = 0; j< g_mapDimension - 1; j++)
			{
				glBegin(GL_TRIANGLE_STRIP);
					setColor(map[i+1][j]);
					glVertex3f((i+1)/scale - 1.0, map[i+1][j], j/scale - 1.0);

					setColor(map[i][j]);
					glVertex3f((i)/scale - 1.0, map[i][j], j/scale - 1.0);

					setColor(map[i+1][j+1]);
					glVertex3f((i+1)/scale - 1.0, map[i+1][j+1], (j+1)/scale - 1.0);

					setColor(map[i][j+1]);
					glVertex3f((i)/scale - 1.0, map[i][j+1], (j+1)/scale - 1.0);
				glEnd();
			}
		}
	/* restores previously saved scalee */
	glPopMatrix();
	/* make sure all previously issued commands are completed */
    glFlush();
	/* contents of back buffer are set to become that of front buffer */
	glutSwapBuffers();
}

/*
=================
keys
-Handles all normal keyboard input
=================
*/
void keys(unsigned char key, int x, int y)
{
	switch(key)
	{
		case 'e': case 'E':
		{
			if((g_horisontal -= 15) < 0)
				g_horisontal += 360;
		}
		break;

		case 'q': case 'Q':
		{
			if((g_horisontal += 15) > 360)
				g_horisontal -= 360;
		}
		break;

		case 'w': case 'W':
		{
			if((g_vertical -= 15) < 0)
				g_vertical += 360;
		}
		break;

		case 's': case 'S':
		{
			if((g_vertical += 15) > 360)
				g_vertical -= 360;
		}
		break;

		case 27:
		{
			exit(0);
		}
		break;
	}
	/* sends signal to redraw */
	glutPostRedisplay();
}

/*
=================
myinit
-Handles basic initialisation and setup of OpenGL parameters.
=================
*/
void myinit()
{
	/* enable hidden-surface removal */
	glEnable(GL_DEPTH_TEST);
	/* specify the value used for depth buffer comparisons
	   - GL_LEQUAL: if incoming z value is <= to the stored z value */
	glDepthFunc(GL_LEQUAL);
}

/*
=================
reshape
-Rendered window's reshape function - manages changes to size of the window, correctly adjusting aspect ratio.
=================
*/
void reshape(int w, int h)
{
	/* sets the viewport with the first two parameters the lower-left corner of the viewport
	   rectangle, in pixels and the last two, the width and height of the viewport */
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);

	/* specifies the Projection matrix as the current matrix */
	glMatrixMode(GL_PROJECTION);
	/* move to the center of the screen */
	glLoadIdentity();

	/* sets up the  perspective projection matrix:
	   - the first parameter specifies the field of view (fov), an angle in degrees
	     (in the y-direction)
	   - the second parameter is aspect ratio determining fov
		 (in the x-direction)
	   - the third parameter is the distance from the viewer to the near clipping plane
	   - the last parameter is the distance from the viewer to the far clipping plane */
	gluPerspective(45, (GLdouble)w / (GLdouble)h, 0.05, 100);
	/* specifies the Modelview matrix as the current matrix */
	glMatrixMode(GL_MODELVIEW);
}

/*
=================
Main
-Creates the OpenGL window, sets up OpenGL parameters and calls the display function.
=================
*/
int main(int argc, char **argv)
{
    glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitWindowSize(g_windowWidth, g_windowHeight);
	glutCreateWindow("A height map representing a hat function");
    glutReshapeFunc(reshape);
    glutDisplayFunc(display);
    glutKeyboardFunc(keys);
	myinit();
	setupHeights();
    glutMainLoop();
}
//EOF